/// \file CncStepper.cpp
/// \brief Simple stepper motor controller representing one channel of the CNC Shield.
/// \copyright Copyright (c) 2015, Garth Zeglin.  All rights reserved. Licensed
///            under the terms of the BSD 3-clause license.

#include "CncStepper.h"
#include <Arduino.h>

CncStepper::CncStepper( uint8_t _step_pin, uint8_t _dir_pin )
{
  step_pin = _step_pin;
  dir_pin  = _dir_pin;
  position = 0;
  target   = 0;
  elapsed  = 0;
  
  step_interval = 200;  // 200 microseconds = 5000 steps/sec
}

void CncStepper::pollForInterval(unsigned long interval)
{
  // Accumulated the time elapsed since the step.
  elapsed += interval;

  if (elapsed >= step_interval) {
    // reset the timer according to the target interval to produce a correct an
    // average rate even if extra time has passed
    elapsed -= step_interval;

    // check whether to emit a step
    if ( position != target ) {

      // always set the direction to match the sign of motion
      digitalWrite( dir_pin, (position < target) ? HIGH : LOW );

      // emit a step
      digitalWrite( step_pin, HIGH );

      // update the position count
      if (position < target) position++;
      else position--;
      digitalWrite( step_pin, LOW );
    }
  }
}
			 
